
from csp import *
import time

#______________________________________________________________________________
# Coloring Problems

def different_values_constraint_plus(A, a, B, b):
    "A constraint saying two neighboring variables must differ in value."
    areaCsp.consistencyChecks += 1
    return a != b

##------------------------------Map Coloring CSP Class------------------------------

class MapColoring(CSP):
    """Make a CSP for the problem of coloring a map with different colors
    for any two adjacent regions.  Arguments are a list of colors, and a
    dict of {region: [neighbor,...]} entries.  This dict may also be
    specified as a string of the form defined by parse_neighbors."""
    
    def __init__(self, colors, neighbors) :

        self.consistencyChecks = 0
        
        if isinstance(neighbors, str):
            neighbors = parse_neighbors(neighbors)
            
        CSP.__init__(self, neighbors.keys(), UniversalDict(colors), neighbors,
                       different_values_constraint_plus)
        

    def solve(self) :    
        #FC+MRV algorith
        start_time = time.time()

        result = backtracking_search(self, select_unassigned_variable=mrv, inference=forward_checking)

        print "--FC+MRV--\nTime elapsed: {0} sec\nNumber of assignments: {1}\nNumber of consistency checks: {2}\n".format(
                time.time() - start_time, self.nassigns, self.consistencyChecks)
        #MAC algorith
        start_time = time.time()
        
        result = backtracking_search(self, inference=mac)

        print "--MAC--\nTime elapsed: {0} sec\nNumber of assignments: {1}\nNumber of consistency checks: {2}\n".format(
                time.time() - start_time, self.nassigns, self.consistencyChecks)

        #Min-Conflict algorith
        start_time = time.time()
        
        result = min_conflicts(self)

        print "--Min-Conflict--\nTime elapsed: {0} sec\nNumber of assignments: {1}\nNumber of consistency checks: {2}\n".format(
                time.time() - start_time, self.nassigns, self.consistencyChecks)  

##------------------------------Map Coloring Examples------------------------------

au = MapColoring(list('RGB'),
                           'SA: WA NT Q NSW V; NT: WA Q; NSW: Q V; T: ')

us = MapColoring(list('RGBY'),
        """WA: OR ID; OR: ID NV CA; CA: NV AZ; NV: ID UT AZ; ID: MT WY UT;
        UT: WY CO AZ; MT: ND SD WY; WY: SD NE CO; CO: NE KA OK NM; NM: OK TX;
        ND: MN SD; SD: MN IA NE; NE: IA MO KA; KA: MO OK; OK: MO AR TX;
        TX: AR LA; MN: WI IA; IA: WI IL MO; MO: IL KY TN AR; AR: MS TN LA;
        LA: MS; WI: MI IL; IL: IN KY; IN: OH KY; MS: TN AL; AL: TN GA FL;
        MI: OH IN; OH: PA WV KY; KY: WV VA TN; TN: VA NC GA; GA: NC SC FL;
        PA: NY NJ DE MD WV; WV: MD VA; VA: MD DC NC; NC: SC; NY: VT MA CT NJ;
        NJ: DE; DE: MD; MD: DC; VT: NH MA; MA: NH RI CT; CT: RI; ME: NH;
        HI: ; AK: """)

fr = MapColoring(list('RGBY'),
        """AL: LO FC; AQ: MP LI PC; AU: LI CE BO RA LR MP; BO: CE IF CA FC RA
        AU; BR: NB PL; CA: IF PI LO FC BO; CE: PL NB NH IF BO AU LI PC; FC: BO
        CA LO AL RA; IF: NH PI CA BO CE; LI: PC CE AU MP AQ; LO: CA AL FC; LR:
        MP AU RA PA; MP: AQ LI AU LR; NB: NH CE PL BR; NH: PI IF CE NB; NO:
        PI; PA: LR RA; PC: PL CE LI AQ; PI: NH NO CA IF; PL: BR NB CE PC; RA:
        AU BO FC PA LR""")

#______________________________________________________________________________
# Main

print "################ Australia ################"
areaCsp = au
areaCsp.solve()

print "################ USA ################"
areaCsp = us
areaCsp.solve()

print "################ France ################"
areaCsp = fr
areaCsp.solve()
